
; Disk Driver Specification ver. 2.00
;[]===========================================================[]
;Procedure : Initialization
;
;Function  : Initialization device(s)
;
;Input	   : C = 00h
;	    IX - Environment
;Output	   : A = Amount	drive support
;	    HL = Size driver
;[]===========================================================[]
;[]===========================================================[]
;Procedure : Open
;
;Function  : Open disk
;
;Input	   : C = 01h
;	     A - Drive
;Output	   : None
;
;[]===========================================================[]
;[]===========================================================[]
;Procedure : Close
;
;Function  : Close disk
;
;Input	   : C = 02h
;	     A - Drive
;Output	   : None
;
;[]===========================================================[]
;[]===========================================================[]
;Procedure : Media check
;
;Function  : Checking change line
;
;Input	   : C = 03h
;	     A - Drive
;Output	   : A = 00h disk no changed
;	     A = 0FFh disk changed
;
;[]===========================================================[]
;[]===========================================================[]
;Procedure : Get BPB
;
;Function  : Get Block Parameters BIOS
;
;Input	   : C = 04h
;	    DE - Address
;Output	   : None
;
;[]===========================================================[]
;[]===========================================================[]
;Procedure : Input
;
;Function  : Input from	disk
;
;Input	   : C = 05h
;	 HL:IX - Logical Block (sector)
;	    DE - Address
;	     B - Sector	count
;Output	   : None
;
;[]===========================================================[]
;[]===========================================================[]
;Procedure : Output
;
;Function  : Output to disk
;
;Input	   : C = 06h
;	 HL:IX - Logical Block (sector)
;	    DE - Address
;	     B - Sector	count
;Output	   : None
;
;[]===========================================================[]
;[]===========================================================[]
;Procedure : Removable
;
;Function  : Checking change line
;
;Input	   : C = 07h
;	     A - Drive
;Output	   : A = 00h Removable
;	     A = FFh Nonremovable
;
;[]===========================================================[]
;[]===========================================================[]
;Procedure : Generic IOCTL
;
;Function  : Generic Input Output Control
;
;Input	   : C = 08h
;	     B - Subcommand
;	    DE = 55AAh Magic Number
;      Subcommand
;----------------------
;	  00h -	Get Device Parameters
;	  01h -	Read track
;	  02h -	Test track
;	  80h -	Set Device Parameters
;	  81h -	Write track
;	  82h -	Format track
;Output	   :
;
;[]===========================================================[]
;
; Errors:
;   0 (00h) - NO ERRORS
;   1 (01h) - BAD COMMAND
;   2 (02h) - BAD DRIVE	NUMBER
;   3 (03h) - UNKNOW FORMAT
;   4 (04h) - NOT READY
;   5 (05h) - SEEK ERROR
;   6 (06h) - SECTOR NOT FOUND
;   7 (07h) - CRC ERROR
;   8 (08h) - WRITE PROTECT
;   9 (09h) - READ ERROR
;  10 (0Ah) - WRITE ERROR
;  11 (0Bh) - FAILURE
;  12 (0Ch) - BUSY (DEVICE OPENED)
;  13 (0Dh) - RESERVED

R_COM	EQU	#0F	;Command/Status
R_TRK	EQU	#3F	;Track
R_SEC	EQU	#5F	;Sector
R_DAT	EQU	#7F	;Data
R_DSK	EQU	#FF	;Drive Control

COM_B	EQU	#1B

FDDRIVE	INC	C
	DEC	C
	JP	Z,INIT
	DEC	C
	JP	Z,RESE
	DEC	C
	JP	Z,STAT
	DEC	C
	JP	Z,CHEK
	DEC	C
	JP	Z,GBPB
	DEC	C
	JP	Z,READD
	DEC	C
	JP	Z,WRITED
	LD	A,1
	SCF 
	RET 

;Commands for restart #18
INIT	LD	A,2
	AND	A
	RET 

RESE	CALL	DOS_ON
	CALL	S_FDD
	CALL	DISK_ID
	LD	A,4	;NOT READY /NO DISK/
	RET	C
	CALL	RESWG
	XOR	A
	OUT	(R_TRK),A
	IN	A,(R_COM)
	LD	C,A
	CALL	DOS_OFF
	LD	A,C
	AND	A
	RET 

STAT	CALL	DOS_ON
	IN	A,(R_COM)
	CALL	DOS_OFF
	AND	#7F
	RET	Z
	SCF 
	RET 

CHEK	LD	A,#FF
	AND	A
	RET 

;DE - ADDRESS

GBPB	CALL	DOS_ON
	CALL	S_FDD
	CALL	DISK_ID
	LD	A,4	;NOT READY /NO DISK/
	RET	C
	CALL	RESWG
	XOR	A
	OUT	(R_TRK),A
	LD	B,1
	EX	DE,HL
	LD	DE,#0000
	PUSH	HL
	CALL	DSK_LP
	EX	AF,AF'
	EXX 
	POP	HL
	LD	BC,#0018
	ADD	HL,BC
	LD	A,(HL)
	LD	(MS_SECS),A
	EXX 
	EX	AF,AF'
	RET 

TURNSPD	LD	A,(F144)
	XOR	1
	LD	(F144),A
SPEED	LD	A,(F144)
	AND	1
	JP	Z,RES144
	JP	SET144	;NZ

DISK_ID	DI 
	EXX 
	CALL	SPEED
	IN	A,(R_TRK)
	OUT	(R_DAT),A
	LD	A,#18	;SEARCH
	CALL	EXECOM
	LD	C,4
ID_LP0	LD	A,#C0
	OUT	(R_COM),A
	LD	HL,#F000
ID_LP1	IN	A,(R_DSK)
	AND	#C0
	JR	Z,ID_LP4
ID_LP2	IN	A,(R_DAT)
ID_LP3	IN	A,(R_DSK)
	AND	#C0
	JR	Z,ID_LP3
	JP	P,ID_LP2
	EXX 
	AND	A
	EI 
	RET 

ID_LP4	DEC	HL
	LD	A,H
	OR	L
	JP	NZ,ID_LP1
	CALL	TURNSPD
	DEC	C
	JP	NZ,ID_LP0
	EXX 
	SCF 
	EI 
	RET 

READD	CALL	DOS_ON
	CALL	S_FDD
	PUSH	DE
	PUSH	HL
	PUSH	IX
	POP	DE
	POP	IX
	PUSH	BC
	CALL	NTRACK
	POP	BC
	POP	HL
	LD	A,B
	OR	A
	RET	Z
DSK_LP	LD	A,D
	EXX 
	CALL	G_trk
	EXX 
	PUSH	BC
	PUSH	DE
	PUSH	HL
	CALL	RD_SEC
	LD	DE,(B_P_S)
	POP	HL
	JP	C,ERRDOS
	ADD	HL,DE
	POP	DE
	POP	BC
	LD	A,(MS_SECS)
	INC	E
	CP	E
	JP	NZ,NINC_T
	LD	E,0
	INC	D
NINC_T	DJNZ	DSK_LP
RETDOS	CALL	DOS_OFF
	XOR	A
	RET 

ERRDOS	POP	DE
	POP	BC
	EX	AF,AF'
	CALL	DOS_OFF
	EX	AF,AF'
	SCF 
	RET 

WRITED	CALL	DOS_ON
	CALL	S_FDD
	PUSH	DE
	PUSH	HL
	PUSH	IX
	POP	DE
	POP	IX
	PUSH	BC
	CALL	NTRACK
	POP	BC
	POP	HL
	LD	A,B
	OR	A
	RET	Z
DSK_LP2	LD	A,D
	EXX 
	CALL	G_trk
	EXX 
	PUSH	BC
	PUSH	DE
	PUSH	HL
	CALL	WR_SEC
	LD	DE,(B_P_S)
	POP	HL
	JP	C,ERRDOS
	ADD	HL,DE
	POP	DE
	POP	BC
	LD	A,(MS_SECS)
	INC	E
	CP	E
	JP	NZ,NINC_T2
	LD	E,0
	INC	D
NINC_T2	DJNZ	DSK_LP2
	CALL	DOS_OFF
	XOR	A
	RET 

S_FDD	LD	(DISK),A
	OR	#3C
	OUT	(R_DSK),A
	RET 

RD_SEC	LD	D,5	;RETRY COUNT
RRETRY	DI 
	PUSH	DE
	LD	A,E
	INC	A
	OUT	(R_SEC),A
	CALL	FDREAD
	POP	DE
	EI 
	IN	A,(R_COM)
	LD	C,A
	AND	#7F
	RET	Z
	BIT	2,C
	JR	NZ,ERDATA
	LD	A,5	;SEEK ERROR
	DEC	D
	JP	Z,RSTOP
	PUSH	DE
	CALL	RESWG		; RESET_WG
	LD	A,(RW_TRK)
	CALL	G_trk	   ; !!!!!!!!!!!!!!!!
	POP	DE
	JR	RRETRY
ERDATA	DEC	D	       ; POTERIA DANNYH
	JR	NZ,RRETRY
	LD	A,6	;SECTOR	NOT FOUND
RSTOP	EX	AF,AF'
	LD	A,#D0
	OUT	(R_COM),A      ;STOP OPERATION
	EX	AF,AF'
	BIT	0,C
	SCF 
	RET	Z
	LD	A,9	;READ ERROR /NO	DISK/
	RET 

WR_SEC	LD	D,5	;RETRY COUNT
WRETRY	DI 
	PUSH	DE
	LD	A,E
	INC	A
	OUT	(R_SEC),A
	CALL	FDWRITE
	POP	DE
	EI 
	IN	A,(R_COM)
	LD	C,A
	AND	#7F
	RET	Z
	BIT	6,C
	LD	A,8	;WRITE PROTECT
	JR	NZ,WSTOP
	BIT	2,C
	JR	NZ,EWDATA
	LD	A,5	;SEEK ERROR
	DEC	D
	JP	Z,WSTOP
	PUSH	DE
	CALL	RESWG		; RESET_WG
	LD	A,(RW_TRK)
	CALL	G_trk	   ; !!!!!!!!!!!!!!!!
	POP	DE
	JR	WRETRY
EWDATA	DEC	D	       ; POTERIA DANNYH
	JR	NZ,WRETRY
	LD	A,6	;SECTOR	NOT FOUND
WSTOP	EX	AF,AF'
	LD	A,#D0
	OUT	(R_COM),A      ;STOP OPERATION
	EX	AF,AF'
	BIT	0,C
	SCF 
	RET	Z
	LD	A,10	;WRITE ERROR
	RET 

;-------------------------------
;READ SECTOR
;-------------------------------
FDREAD	LD	B,4
	LD	C,R_DAT
	LD	A,#80		;COMMAND READ
	OUT	(R_COM),A
FDR001	IN	A,(R_DSK)	;WAIT INTRQ or DRQ
	AND	#C0
	JR	Z,FDR003
FDR004	INI			;READ BYTE
FDR002	IN	A,(R_DSK)
	AND	#C0
	JR	Z,FDR002
	JP	P,FDR004
	RET 

FDR003	INC	DE
	LD	A,E
	OR	D
	JR	NZ,FDR001
	DJNZ	FDR001
	RET 

;-------------------------------
;WRITE SECTOR
;-------------------------------

FDWRITE	LD	B,4
	LD	C,R_DAT
	LD	A,#A0		;COMMAND WRITE
	OUT	(R_COM),A
FDW001	IN	A,(R_DSK)	;WAIT INTRQ or DRQ
	AND	#C0
	JR	Z,FDW003
FDW004	OUTI			;WRITE BYTE
FDW002	IN	A,(R_DSK)
	AND	#C0
	JR	Z,FDW002
	JP	P,FDW004
	RET 

FDW003	INC	DE
	LD	A,E
	OR	D
	JR	NZ,FDW001
	DJNZ	FDW001
	RET 

G_trk	LD	(RW_TRK),A
	LD	C,A
	SRL	C
	LD	A,(DISK)
	JR	C,GT001
	OR	#3C
GT001	OR	#2C
	OUT	(R_DSK),A
	IN	A,(R_TRK)
	CP	C
	PUSH	BC
	CALL	NZ,P50ms
	POP	BC
	LD	A,C
	OUT	(R_DAT),A
	IN	A,(R_TRK)
	CP	C
	EX	AF,AF'	;R.TRACK==PHISICAL TRACK
	LD	A,#18
	CALL	EXECOM
	RET	C
	EX	AF,AF'
	LD	A,C
	OUT	(R_TRK),A
	RET	Z
STOL	PUSH	BC
	CALL	P50ms
	POP	BC
	RET 

P50ms	LD	A,12
P1ms	LD	C,255
PMS	DEC	C
	JR	NZ,PMS
	DEC	A
	JR	NZ,P1ms
	RET 

RESWG	LD	A,8
EXECOM	OUT	(R_COM),A
	LD	HL,#0000
WREST	DEC	HL
	LD	A,H
	OR	L
	SCF 
	RET	Z
	IN	A,(R_DSK)
	AND	#80
	JR	Z,WREST
	AND	A
	RET 

;-------------------------------

; IX:DE	- SECTOR

; D - TRACK, E - SECTOR

NTRACK	EX	DE,HL
	LD	A,(MS_SECS)
	LD	C,A
	LD	B,0
	LD	DE,#FFFF
	AND	A
NTRK	INC	DE
	SBC	HL,BC
	JP	NC,NTRK
	LD	A,LX
	OR	HX
	JP	Z,LTRK
	DEC	IX
	JP	NTRK
LTRK	ADD	HL,BC
	LD	D,E
	LD	E,L
	RET 

DOS_ON	PUSH	AF
	PUSH	BC
	LD	B,A
	LD	A,(#3D10)
	LD	C,A
	LD	A,#C9
	LD	(#3D10),A
	CALL	#3D10
	LD	A,C
	LD	(#3D10),A
	LD	A,B
	POP	BC
	POP	AF
	RET 

DOS_OFF	PUSH	AF
	PUSH	BC
	LD	B,A
	LD	A,(#4100)
	LD	C,A
	LD	A,#C9
	LD	(#4100),A
	CALL	#4100
	LD	A,C
	LD	(#4100),A
	LD	A,B
	POP	BC
	POP	AF
	RET 

RES144
	LD	A,5
	OUT	(COM_B),A
	LD	A,#E0
	OUT	(COM_B),A
	RET 
SET144
	LD	A,5
	OUT	(COM_B),A
	LD	A,#60
	OUT	(COM_B),A
	RET 

DISK	DEFB	#01
MS_SECS	DEFB	#09
RW_TRK	DEFB	#00
F144	DEFB	#00
;==============================================



